/*
	privss - A private stream searching toolkit.

	Copyright (C) 2006 John Bethencourt.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.
*/

/*
	Include stdlib.h, stdint.h, and gmp.h before including this file.

	Routines to solve a system of linear equations mod n using LUP
	decomposition, somewhat specialized to our application.
*/

/*
	Solve the linear system Ax = b for x given A and b, assuming
	arithmetic mod n.

  A should be a len * len length byte array; x and b should be len
	length arrays of initialized mpz_t's. This routine will return 0 if
	the system is underdetermined (i.e., if A is singular); otherwise it
	will fill in x and return 1.
*/

int solve_system( uint8_t* A, mpz_t* x, mpz_t* b, int len, mpz_t n );

/*
	Decompose a len by len matrix A into a unit lower triangular matrix
	L, an upper triangular matrix U, and a permutation matrix P such
	that PA = LU.

  Assumes A is given in LU (to be operated on in place). Returns 0 if
	A is singular, otherwise returns 0 and fills in LU and P. P is
	represented not as a matrix, but as a permutation of the list
	[0,len-1]. LU consists of the lower triangle of L and the diagonal
	and upper triangle of U crammed into one matrix (not the product of
	L and U).
*/

int lup_decomp( mpz_t* LU, int* P, int len, mpz_t n );

/*
	Solve the linear system Lx = b mod n where L is a unit lower
	triangular matrix.

  Ignores all entries in L on the diagonal (assumed to be one) and in
	the upper triangle (assumed to be zero).
*/

void forward_subst( mpz_t* L, mpz_t* x, mpz_t* b, int len, mpz_t n );

/*
	Solve the linear system Ux = b mod n where U is an upper triangular
	matrix.

  Ignores all entries in U in the lower triangle (since they are
	assumed to be zero).
*/

void back_subst( mpz_t* U, mpz_t* x, mpz_t* b, int len, mpz_t n );

/*
	Convenience functions to allocate and free arrays of mpz_t's.
*/

mpz_t* alloc_mpz_array( int len );
void free_mpz_array( mpz_t* a, int len );
