dnl Check for ATLAS (specifically, cblas.h, libcblas, and libatlas)
dnl and set ATLAS_CFLAGS and ATLAS_LIBS appropriately.

AC_DEFUN([PRIVSS_ATLAS_CHECK],
[

AC_ARG_WITH(atlas-include,
 AC_HELP_STRING([--with-atlas-include=<path>],
   [look for the ATLAS header cblas.h in <path> rather than the
    default search path]),
  [
   ATLAS_INPUT_HEADER=$withval
   ATLAS_HEADER_PATH="${ATLAS_INPUT_HEADER} /usr/include /usr/local/include"
  ],
  [
    ATLAS_HEADER_PATH="/usr/include /usr/local/include"
  ])

AC_ARG_WITH(atlas-lib,
 AC_HELP_STRING([--with-atlas-lib=<path>],
   [look for the ATLAS libraries (libcblas and libatlas) in <path>
    rather than the default search path]),
  [
    ATLAS_INPUT_LIB=$withval
    ATLAS_LIBRARY_PATH="${ATLAS_INPUT_LIB} /lib /usr/lib /usr/local/lib /var/lib"
  ],
  [
    ATLAS_LIBRARY_PATH="/lib /usr/lib /usr/local/lib /var/lib"
  ])

dnl input data check
if test  "x${ATLAS_INPUT_HEADER}" != x; then
  if test -z "${ATLAS_INPUT_LIB}"; then
    echo 'error: since you have specified the ATLAS header path, you must specify the'
    echo 'ATLAS library path using --with-atlas-lib.'
    exit 1
  fi
fi

if test "x${ATLAS_INPUT_LIB}" != x; then
  if test -z "${ATLAS_INPUT_HEADER}"; then
    echo 'error: since you have specified the ATLAS library path, you must specify the'
    echo 'ATLAS header path using --with-atlas-include.'
    exit 1
  fi
fi

dnl check for existence
BACKUP_CFLAGS=${CFLAGS}
BACKUP_LIBS=${LIBS}

AC_MSG_CHECKING(for ATLAS)

atlas_found="no"

for ATLAS_HEADER in ${ATLAS_HEADER_PATH}
do
  if test -r "${ATLAS_HEADER}/cblas.h"; then
    if test "x${ATLAS_HEADER}" != "x/usr/include" -a "x${ATLAS_HEADER}" != "x/usr/local/include"; then
      for ATLAS_LIBRARY in ${ATLAS_LIBRARY_PATH}
      do
        if test "x${ATLAS_LIBRARY}" != "x/lib" -a \
                "x${ATLAS_LIBRARY}" != "x/usr/lib" -a \
                "x${ATLAS_LIBRARY}" != "x/usr/local/lib" -a \
                "x${ATLAS_LIBRARY}" != "x/var/lib"; then
          ATLAS_CFLAGS="-I${ATLAS_HEADER}"
          ATLAS_LIBS="${ATLAS_LIBS} -L${ATLAS_LIBRARY} -lcblas -latlas"
        fi
      done
    else
      ATLAS_CFLAGS=
      ATLAS_LIBS="-lcblas -latlas"
    fi

    CFLAGS="${BACKUP_CFLAGS} ${ATLAS_CFLAGS} ${GMP_CFLAGS}"
    LIBS="${BACKUP_LIBS} ${ATLAS_LIBS} ${GMP_LIBS}"

    AC_TRY_LINK(
    [#include <cblas.h>],
    [double a;],
    [
         atlas_found="yes"
         break
    ],
    [
       unset ATLAS_CFLAGS
       unset ATLAS_LIBS
    ])
  fi
done

if test "x$atlas_found" = "xyes" ; then
  AC_SUBST(ATLAS_CFLAGS)
  AC_SUBST(ATLAS_LIBS)
  AC_DEFINE(HAVE_ATLAS,1,[Define if ATLAS is installed])
  AC_DEFINE(BLAS_AVAILABLE,,[Define if BLAS routines are available])
  HAVE_ATLAS=yes
  AC_MSG_RESULT(found)
else
  AC_MSG_RESULT(not found)
  AC_MSG_ERROR([

The ATLAS library was not found on your system! Please obtain it from
http://math-atlas.sourceforge.net and install it before trying again.
If ATLAS is already installed in a non-standard location, try again
with

  ./configure --with-atlas-include=<path> --with-atlas-lib=<path>

See ./configure --help for more information.
])
fi

CFLAGS=${BACKUP_CFLAGS}
LIBS=${BACKUP_LIBS}
])
