dnl Check for IML and set IML_CFLAGS and IML_LIBS appropriately.

AC_DEFUN([PRIVSS_IML_CHECK],
[

AC_ARG_WITH(iml-include,
 AC_HELP_STRING([--with-iml-include=<path>],
  [look for the header iml.h in <path> rather than the default
   search path]),
  [
   IML_INPUT_HEADER=$withval
   IML_HEADER_PATH="${IML_INPUT_HEADER} /usr/include /usr/local/include"
  ],
  [
    IML_HEADER_PATH="/usr/include /usr/local/include"
  ])

AC_ARG_WITH(iml-lib,
 AC_HELP_STRING([--with-iml-lib=<path>],
   [look for libiml.a in <path> rather than the default
    search path]),
  [
    IML_INPUT_LIB=$withval
    IML_LIBRARY_PATH="${IML_INPUT_LIB} /lib /usr/lib /usr/local/lib /var/lib"
  ],
  [
    IML_LIBRARY_PATH="/lib /usr/lib /usr/local/lib /var/lib"
  ])

dnl input data check
if test  "x${IML_INPUT_HEADER}" != x; then
  if test -z "${IML_INPUT_LIB}"; then
    echo 'error: since you have specified the libiml header path, you must specify the'
    echo 'libiml library path using --with-iml-lib.'
    exit 1
  fi
fi

if test "x${IML_INPUT_LIB}" != x; then
  if test -z "${IML_INPUT_HEADER}"; then
    echo 'error: since you have specified the libiml library path, you must specify the'
    echo 'libiml header path using --with-iml-include.'
    exit 1
  fi
fi

dnl check for existence
BACKUP_CFLAGS=${CFLAGS}
BACKUP_LIBS=${LIBS}

AC_MSG_CHECKING(for IML)

iml_found="no"

for IML_HEADER in ${IML_HEADER_PATH}
do
  if test -r "${IML_HEADER}/iml.h"; then
    if test "x${IML_HEADER}" != "x/usr/include" -a "x${IML_HEADER}" != "x/usr/local/include"; then
      for IML_LIBRARY in ${IML_LIBRARY_PATH}
      do
        if test "x${IML_LIBRARY}" != "x/lib" -a \
                "x${IML_LIBRARY}" != "x/usr/lib" -a \
                "x${IML_LIBRARY}" != "x/usr/local/lib" -a \
                "x${IML_LIBRARY}" != "x/var/lib"; then
          IML_CFLAGS="-I${IML_HEADER}"
          IML_LIBS="${IML_LIBS} -L${IML_LIBRARY} -liml ${GMP_LIBS} ${ATLAS_LIBS}"
        fi
      done
    else
      IML_CFLAGS=
      IML_LIBS="-liml ${GMP_LIBS} ${ATLAS_LIBS}"
    fi

    CFLAGS="${CFLAGS} ${IML_CFLAGS}"
    LIBS="${LIBS} ${IML_LIBS}"

    AC_TRY_LINK(
    [#include <gmp.h>
     #include <iml.h>],
    [long int a;
     mpz_t b;
     mpz_t c;
     mpz_t d;
     nonsingSolvMM(RightSolu, 1, 1, &a, &b, &c, d);],
    [
    AC_MSG_RESULT(found)
    iml_found="yes"
    AC_SUBST(IML_CFLAGS)
    AC_SUBST(IML_LIBS)
    AC_DEFINE(HAVE_IML,1,[Define if IML is installed])
    break
    ],[
    unset IML_CFLAGS
    unset IML_LIBS
    ])
  fi
done

if test "x$iml_found" = "xno"; then
  AC_MSG_RESULT(not found)
  AC_MSG_ERROR([

The Integer Matrix Library (libiml) was not found on your system!
Please obtain it from

  http://www.scg.uwaterloo.ca/~z4chen/iml.html

and install it before trying again. If IML is already installed in a
non-standard location, try again with

  ./configure --with-iml-include=<path> --with-iml-lib=<path>

See ./configure --help for more information.
])
fi

CFLAGS=${BACKUP_CFLAGS}
LIBS=${BACKUP_LIBS}

])

