dnl Check for libpaillier and set PAILLIER_CFLAGS and PAILLIER_LIBS
dnl appropriately.

AC_DEFUN([PRIVSS_PAILLIER_CHECK],
[

AC_ARG_WITH(paillier-include,
 AC_HELP_STRING([--with-paillier-include=<path>],
   [look for the header paillier.h in <path> rather than the default
    search path]),
  [
   PAILLIER_INPUT_HEADER=$withval
   PAILLIER_HEADER_PATH="${PAILLIER_INPUT_HEADER} /usr/include /usr/local/include"
  ],
  [
    PAILLIER_HEADER_PATH="/usr/include /usr/local/include"
  ])

AC_ARG_WITH(paillier-lib,
 AC_HELP_STRING([--with-paillier-lib=<path>],
   [look for libpaillier.a in <path> rather than the default
    search path]),
  [
    PAILLIER_INPUT_LIB=$withval
    PAILLIER_LIBRARY_PATH="${PAILLIER_INPUT_LIB} /lib /usr/lib /usr/local/lib /var/lib"
  ],
  [
    PAILLIER_LIBRARY_PATH="/lib /usr/lib /usr/local/lib /var/lib"
  ])

dnl input data check
if test  "x${PAILLIER_INPUT_HEADER}" != x; then
  if test -z "${PAILLIER_INPUT_LIB}"; then
    echo 'error: since you have specified the libpaillier header path, you must specify the'
    echo 'libpaillier library path using --with-paillier-lib.'
    exit 1
  fi
fi

if test "x${PAILLIER_INPUT_LIB}" != x; then
  if test -z "${PAILLIER_INPUT_HEADER}"; then
    echo 'error: since you have specified the libpaillier library path, you must specify the'
    echo 'libpaillier header path using --with-paillier-include.'
    exit 1
  fi
fi

dnl check for existence
BACKUP_CFLAGS=${CFLAGS}
BACKUP_LIBS=${LIBS}

AC_MSG_CHECKING(for libpaillier)

paillier_found="no"

for PAILLIER_HEADER in ${PAILLIER_HEADER_PATH}
do
  if test -r "${PAILLIER_HEADER}/paillier.h"; then
    if test "x${PAILLIER_HEADER}" != "x/usr/include" -a "x${PAILLIER_HEADER}" != "x/usr/local/include"; then
      for PAILLIER_LIBRARY in ${PAILLIER_LIBRARY_PATH}
      do
        if test "x${PAILLIER_LIBRARY}" != "x/lib" -a \
                "x${PAILLIER_LIBRARY}" != "x/usr/lib" -a \
                "x${PAILLIER_LIBRARY}" != "x/usr/local/lib" -a \
                "x${PAILLIER_LIBRARY}" != "x/var/lib"; then
          PAILLIER_CFLAGS="-I${PAILLIER_HEADER}"
          PAILLIER_LIBS="${PAILLIER_LIBS} -L${PAILLIER_LIBRARY} -lpaillier -lgmp"
        fi
      done
    else
      PAILLIER_CFLAGS=
      PAILLIER_LIBS="-lpaillier -lgmp"
    fi

    CFLAGS="${CFLAGS} ${PAILLIER_CFLAGS}"
    LIBS="${LIBS} ${PAILLIER_LIBS}"

    AC_TRY_LINK(
    [#include <gmp.h>
     #include <paillier.h>],
    [paillier_create_enc_zero();],
    [
    AC_MSG_RESULT(found)
    paillier_found="yes"
    AC_SUBST(PAILLIER_CFLAGS)
    AC_SUBST(PAILLIER_LIBS)
    AC_DEFINE(HAVE_PAILLIER,1,[Define if libpaillier is installed])
    break
    ],[
    unset PAILLIER_CFLAGS
    unset PAILLIER_LIBS
    ])
  fi
done

if test "x$paillier_found" = "xno"; then
  AC_MSG_RESULT(not found)
  AC_MSG_ERROR([

The library libpaillier was not found on your system! Please obtain it
from

  http://www.cs.cmu.edu/~bethenco/libpaillier

and install it before trying again. If libpaillier is already
installed in a non-standard location, try again with

  ./configure --with-paillier-include=<path> --with-paillier-lib=<path>

See ./configure --help for more information.
])
fi

CFLAGS=${BACKUP_CFLAGS}
LIBS=${BACKUP_LIBS}

])
